﻿using Microsoft.IoT.Lightning.Providers;
using System;
using System.Diagnostics;
using System.Text;
using System.Threading.Tasks;
using Windows.Devices;
using Windows.Devices.Gpio;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace Remocon
{
	public sealed partial class MainPage : Page
	{
		private GpioPin OpenPin(int pinno, bool input, bool high = false)
		{
			var pin = GpioController.GetDefault().OpenPin(pinno);
			if (input == false) {
				pin.SetDriveMode(GpioPinDriveMode.Output);
				pin.Write((high == true) ? GpioPinValue.High : GpioPinValue.Low);
			} else {
				pin.SetDriveMode(GpioPinDriveMode.Input);
			}
			return pin;
		}

		private GpioPin irpin_in;
		private const int ST_MAX = 512;
		private int[] signalTimes = new int[ST_MAX];
		private Stopwatch sw = new Stopwatch();

		public MainPage()
		{
			InitializeComponent();

			if (LightningProvider.IsLightningEnabled) {
				LowLevelDevicesController.DefaultProvider = LightningProvider.GetAggregateProvider();
			}
			
			irpin_in = OpenPin(17, true);
		}

		private async void SetButtonEnable(bool enable)
		{
			await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => {
				btnDebugger.IsEnabled = enable;
				btnReceive.IsEnabled = enable;
			});
		}

		private const long timeoutMicros = 5000000;
		private const long waitMicros = 1000000;

		private long micros()
		{
			return (long)(sw.Elapsed.TotalMilliseconds * 1000.0);
		}

		private bool waitChange(GpioPinValue status, bool began)
		{
			long waitStartMicros = micros();
			long waits = (began == true) ? timeoutMicros : waitMicros;
			while (irpin_in.Read() == status) {
				if (micros() - waitStartMicros > waits) return true;
			}
			return false;
		}

		private void OnClickReceive(object sender, RoutedEventArgs e)
		{
			SetButtonEnable(false);

			Task.Run(async () => {
				GpioPinValue moduleStatus;
				long now, lastMicros;
				sw.Restart();

				now = micros();
				lastMicros = micros();

				bool began = false;
				int stcount = 0;

				moduleStatus = irpin_in.Read();

				while (stcount < ST_MAX) {
					bool waitStatus = waitChange(moduleStatus, began);
					if (waitStatus == true) {
						if (began == true) {
							break;
						} else {
							continue;
						}
					} else {
						Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => {
							txtSignal.Text = "Analyzing...";
						});
						began = true;
					}

					now = micros();
					signalTimes[stcount++] = (int)((now - lastMicros) / 10);
					lastMicros = now;

					moduleStatus = (moduleStatus == GpioPinValue.High) ? GpioPinValue.Low : GpioPinValue.High;
				}

				await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => {
					StringBuilder sb = new StringBuilder();
					for(int i = 0; i < stcount; i++) {
						sb.AppendFormat("{0},", signalTimes[i]);
					}
					txtSignal.Text = sb.ToString();

					SetButtonEnable(true);
				});
			});
		}

		private void OnClickDebugger(object sender, RoutedEventArgs e)
		{
			Debug.Write(txtSignal.Text);
		}
	}
}
